@@ -17,7 +17,6 @@
 
 use std::{
     collections::HashMap,
-    fmt::Display,
     fs,
     fs::{create_dir_all, File},
     io::prelude::*,
@@ -27,14 +26,18 @@ use std::{
 };
 
 use chrono::prelude::*;
-use git2::{Oid, Repository};
 use totp_rs::TOTP;
 
-use crate::crypto::{
-    Crypto, CryptoImpl, FindSigningFingerprintStrategy, GpgMe, Sequoia, VerificationError,
+use crate::{
+    crypto::{Crypto, CryptoImpl, GpgMe, Sequoia, VerificationError},
+    git::{
+        add_and_commit_internal, commit, find_last_commit, init_git_repo, match_with_parent,
+        move_and_commit, push_password_if_match, read_git_meta_data, remove_and_commit,
+        verify_git_signature,
+    },
 };
 pub use crate::{
-    error::{Error, Result},
+    error::{to_result, Error, Result},
     signature::{
         parse_signing_keys, Comment, KeyRingStatus, OwnerTrustLevel, Recipient, SignatureStatus,
     },
@@ -55,6 +58,8 @@ pub struct PasswordStore {
     style_file: Option<PathBuf>,
     /// The gpg implementation
     crypto: Box<dyn Crypto + Send>,
+    /// The home dir of the user, if it exists
+    user_home: Option<PathBuf>,
 }
 
 impl PasswordStore {
@@ -98,6 +103,7 @@ impl PasswordStore {
             passwords: [].to_vec(),
             style_file: style_file.to_owned(),
             crypto,
+            user_home: home.clone(),
         };
 
         if !store.valid_gpg_signing_keys.is_empty() {
@@ -190,6 +196,7 @@ impl PasswordStore {
             passwords: [].to_vec(),
             style_file: style_file.to_owned(),
             crypto,
+            user_home: home.clone(),
         };
 
         Ok(store)
@@ -210,6 +217,10 @@ impl PasswordStore {
         self.root.clone()
     }
 
+    pub fn get_user_home(&self) -> Option<PathBuf> {
+        self.user_home.clone()
+    }
+
     /// returns the style file for the store
     pub fn get_style_file(&self) -> Option<PathBuf> {
         self.style_file.clone()
@@ -220,7 +231,7 @@ impl PasswordStore {
         &*self.crypto
     }
 
-    fn repo(&self) -> Result<git2::Repository> {
+    pub fn repo(&self) -> Result<git2::Repository> {
         git2::Repository::open(&self.root).map_err(Error::Git)
     }
 
@@ -275,7 +286,7 @@ impl PasswordStore {
                     std::fs::create_dir(&path)?;
                 }
             } else {
-                path.push(format!("{}.gpg", p));
+                path.push(format!("{p}.gpg"));
             }
         }
 
@@ -327,7 +338,7 @@ impl PasswordStore {
                 ))
             }
             Ok(repo) => {
-                let message = format!("Add password for {} using ripasso", path_end);
+                let message = format!("Add password for {path_end} using ripasso");
 
                 add_and_commit_internal(
                     &repo,
@@ -556,7 +567,7 @@ impl PasswordStore {
             .into_iter()
             .map(|s| format!("0x{}, ", s.key_id))
             .collect::<String>();
-        let message = format!("Reencrypt password store with new GPG ids {}", keys);
+        let message = format!("Reencrypt password store with new GPG ids {keys}");
 
         self.add_and_commit(&names, &message)?;
 
@@ -699,46 +710,6 @@ pub fn all_recipients_from_stores(
     Ok(all_recipients)
 }
 
-fn push_password_if_match(
-    target: &Path,
-    found: &Path,
-    commit: &git2::Commit,
-    repo: &git2::Repository,
-    passwords: &mut Vec<PasswordEntry>,
-    oid: &git2::Oid,
-    store: &PasswordStore,
-) -> bool {
-    if *target == *found {
-        let time = commit.time();
-        let time_return = Ok(Local.timestamp(time.seconds(), 0));
-
-        let name_return = name_from_commit(commit);
-
-        let signature_return = verify_git_signature(repo, oid, store);
-
-        passwords.push(PasswordEntry::new(
-            &store.root,
-            target,
-            time_return,
-            name_return,
-            signature_return,
-            RepositoryStatus::InRepo,
-        ));
-        return false;
-    }
-    true
-}
-
-/// Find the name of the committer, or an error message
-fn name_from_commit(commit: &git2::Commit) -> Result<String> {
-    commit
-        .committer()
-        .name()
-        .map_or(Err(Error::Generic("missing committer name")), |s| {
-            Ok(s.to_owned())
-        })
-}
-
 /// Describes one log line in the history of a file
 #[non_exhaustive]
 pub struct GitLogLine {
@@ -906,13 +877,13 @@ impl PasswordEntry {
                 let mut end_pos = secret.len();
                 for (pos, c) in secret.chars().skip(start_pos).enumerate() {
                     if c.is_whitespace() {
-                        end_pos = pos;
+                        end_pos = pos + start_pos;
                         break;
                     }
                 }
                 end_pos
             };
-            let totp = TOTP::<&str>::from_url(&secret[start_pos..end_pos])?;
+            let totp = TOTP::from_url(&secret[start_pos..end_pos])?;
             Ok(totp.generate_current()?)
         } else {
             Err(Error::Generic("No otpauth:// url in secret"))
@@ -1020,7 +991,7 @@ impl PasswordEntry {
                         }
 
                         let time = commit.time();
-                        let dt = Local.timestamp(time.seconds(), 0);
+                        let dt = to_result(Local.timestamp_opt(time.seconds(), 0)).ok()?;
 
                         let signature_status = verify_git_signature(&repo, &oid, store);
                         Some(GitLogLine::new(
@@ -1041,325 +1012,6 @@ impl PasswordEntry {
     }
 }
 
-/// returns true if the diff between the two commit's contains the path that the `DiffOptions`
-/// have been prepared with
-fn match_with_parent(
-    repo: &git2::Repository,
-    commit: &git2::Commit,
-    parent: &git2::Commit,
-    opts: &mut git2::DiffOptions,
-) -> Result<bool> {
-    let a = parent.tree()?;
-    let b = commit.tree()?;
-    let diff = repo.diff_tree_to_tree(Some(&a), Some(&b), Some(opts))?;
-    Ok(diff.deltas().len() > 0)
-}
-
-fn find_last_commit(repo: &git2::Repository) -> Result<git2::Commit> {
-    let obj = repo.head()?.resolve()?.peel(git2::ObjectType::Commit)?;
-    obj.into_commit()
-        .map_err(|_| Error::Generic("Couldn't find commit"))
-}
-
-/// Returns if a git commit should be gpg signed or not.
-fn should_sign(repo: &git2::Repository) -> bool {
-    repo.config().map_or(false, |config| {
-        config.get_bool("commit.gpgsign").unwrap_or(false)
-    })
-}
-
-/// Apply the changes to the git repository.
-fn commit(
-    repo: &git2::Repository,
-    signature: &git2::Signature,
-    message: &str,
-    tree: &git2::Tree,
-    parents: &[&git2::Commit],
-    crypto: &(dyn Crypto + Send),
-) -> Result<git2::Oid> {
-    if should_sign(repo) {
-        let commit_buf = repo.commit_create_buffer(
-            signature, // author
-            signature, // committer
-            message,   // commit message
-            tree,      // tree
-            parents,   // parents
-        )?;
-
-        let commit_as_str = str::from_utf8(&commit_buf)?;
-
-        let sig = crypto.sign_string(commit_as_str, &[], &FindSigningFingerprintStrategy::GIT)?;
-
-        let commit = repo.commit_signed(commit_as_str, &sig, Some("gpgsig"))?;
-
-        if let Ok(mut head) = repo.head() {
-            head.set_target(commit, "added a signed commit using ripasso")?;
-        } else {
-            repo.branch(&git_branch_name(repo)?, &repo.find_commit(commit)?, false)?;
-        }
-
-        Ok(commit)
-    } else {
-        let commit = repo.commit(
-            Some("HEAD"), //  point HEAD to our new commit
-            signature,    // author
-            signature,    // committer
-            message,      // commit message
-            tree,         // tree
-            parents,      // parents
-        )?;
-
-        Ok(commit)
-    }
-}
-
-fn git_branch_name(repo: &git2::Repository) -> Result<String> {
-    let head = repo.find_reference("HEAD")?;
-    let symbolic = head
-        .symbolic_target()
-        .ok_or(Error::Generic("no symbolic target"))?;
-
-    let mut parts = symbolic.split('/');
-
-    Ok(parts
-        .nth(2)
-        .ok_or(Error::Generic(
-            "symbolic target name should be on format 'refs/heads/main'",
-        ))?
-        .to_owned())
-}
-
-/// Add a file to the store, and commit it to the supplied git repository.
-fn add_and_commit_internal(
-    repo: &git2::Repository,
-    paths: &[PathBuf],
-    message: &str,
-    crypto: &(dyn Crypto + Send),
-) -> Result<git2::Oid> {
-    let mut index = repo.index()?;
-    for path in paths {
-        index.add_path(path)?;
-        index.write()?;
-    }
-    let signature = repo.signature()?;
-
-    let mut parents = vec![];
-    let parent_commit;
-    if let Ok(pc) = find_last_commit(repo) {
-        parent_commit = pc;
-        parents.push(&parent_commit);
-    }
-    let oid = index.write_tree()?;
-    let tree = repo.find_tree(oid)?;
-
-    let oid = commit(repo, &signature, message, &tree, &parents, crypto)?;
-
-    Ok(oid)
-}
-
-/// Remove a file from the store, and commit the deletion to the supplied git repository.
-fn remove_and_commit(store: &PasswordStore, paths: &[PathBuf], message: &str) -> Result<git2::Oid> {
-    let repo = store
-        .repo()
-        .map_err(|_| Error::Generic("must have a repository"))?;
-
-    let mut index = repo.index()?;
-    for path in paths {
-        index.remove_path(path)?;
-        index.write()?;
-    }
-    let oid = index.write_tree()?;
-    let signature = repo.signature()?;
-    let parent_commit_res = find_last_commit(&repo);
-    let mut parents = vec![];
-    let parent_commit;
-    if parent_commit_res.is_ok() {
-        parent_commit = parent_commit_res?;
-        parents.push(&parent_commit);
-    }
-    index.write_tree()?;
-    let tree = repo.find_tree(oid)?;
-
-    let oid = commit(
-        &repo,
-        &signature,
-        message,
-        &tree,
-        &parents,
-        store.crypto.as_ref(),
-    )?;
-
-    Ok(oid)
-}
-
-/// Move a file to a new place in the store, and commit the move to the supplied git repository.
-fn move_and_commit(
-    store: &PasswordStore,
-    old_name: &Path,
-    new_name: &Path,
-    message: &str,
-) -> Result<git2::Oid> {
-    let repo = store
-        .repo()
-        .map_err(|_| Error::Generic("must have a repository"))?;
-
-    let mut index = repo.index()?;
-    index.remove_path(old_name)?;
-    index.add_path(new_name)?;
-    index.write()?;
-    let oid = index.write_tree()?;
-    let signature = repo.signature()?;
-    let parent_commit_res = find_last_commit(&repo);
-    let mut parents = vec![];
-    let parent_commit;
-    if parent_commit_res.is_ok() {
-        parent_commit = parent_commit_res?;
-        parents.push(&parent_commit);
-    }
-    let tree = repo.find_tree(oid)?;
-
-    let oid = commit(
-        &repo,
-        &signature,
-        message,
-        &tree,
-        &parents,
-        store.crypto.as_ref(),
-    )?;
-
-    Ok(oid)
-}
-
-/// find the origin of the git repo, with the following strategy:
-/// find the branch that HEAD points to, and read the remote configured for that branch
-/// returns the remote and the name of the local branch
-fn find_origin(repo: &git2::Repository) -> Result<(git2::Remote, String)> {
-    for branch in repo.branches(Some(git2::BranchType::Local))? {
-        let b = branch?.0;
-        if b.is_head() {
-            let upstream_name_buf = repo.branch_upstream_remote(&format!(
-                "refs/heads/{}",
-                &b.name()?.ok_or("no branch name")?
-            ))?;
-            let upstream_name = upstream_name_buf
-                .as_str()
-                .ok_or("Can't convert to string")?;
-            let origin = repo.find_remote(upstream_name)?;
-            return Ok((origin, b.name()?.ok_or("no branch name")?.to_owned()));
-        }
-    }
-
-    Err(Error::Generic("no remotes configured"))
-}
-
-/// function that can be used for callback handling of the ssh interaction in git2
-fn cred(
-    tried_sshkey: &mut bool,
-    _url: &str,
-    username: Option<&str>,
-    allowed: git2::CredentialType,
-) -> std::result::Result<git2::Cred, git2::Error> {
-    let sys_username = whoami::username();
-    let user: &str = username.map_or(&sys_username, |name| name);
-
-    if allowed.contains(git2::CredentialType::USERNAME) {
-        return git2::Cred::username(user);
-    }
-
-    if *tried_sshkey {
-        return Err(git2::Error::from_str("no authentication available"));
-    }
-    *tried_sshkey = true;
-
-    git2::Cred::ssh_key_from_agent(user)
-}
-
-/// Push your changes to the remote git repository.
-/// # Errors
-/// Returns an `Err` if the repository doesn't exist or if an git operation fails
-pub fn push(store: &PasswordStore) -> Result<()> {
-    let repo = store
-        .repo()
-        .map_err(|_| Error::Generic("must have a repository"))?;
-
-    let mut ref_status = None;
-    let (mut origin, branch_name) = find_origin(&repo)?;
-    let res = {
-        let mut callbacks = git2::RemoteCallbacks::new();
-        let mut tried_ssh_key = false;
-        callbacks.credentials(|_url, username, allowed| {
-            cred(&mut tried_ssh_key, _url, username, allowed)
-        });
-        callbacks.push_update_reference(|_refname, status| {
-            ref_status = status.map(std::borrow::ToOwned::to_owned);
-            Ok(())
-        });
-        let mut opts = git2::PushOptions::new();
-        opts.remote_callbacks(callbacks);
-        origin.push(&[format!("refs/heads/{}", branch_name)], Some(&mut opts))
-    };
-    match res {
-        Ok(()) if ref_status.is_none() => Ok(()),
-        Ok(()) => Err(Error::GenericDyn(format!(
-            "failed to push a ref: {:?}",
-            ref_status
-        ))),
-        Err(e) => Err(Error::GenericDyn(format!("failure to push: {}", e))),
-    }
-}
-
-/// Pull new changes from the remote git repository.
-/// # Errors
-/// Returns an `Err` if the repository doesn't exist or if an git operation fails
-pub fn pull(store: &PasswordStore) -> Result<()> {
-    let repo = store
-        .repo()
-        .map_err(|_| Error::Generic("must have a repository"))?;
-
-    let (mut origin, branch_name) = find_origin(&repo)?;
-
-    let mut cb = git2::RemoteCallbacks::new();
-    let mut tried_ssh_key = false;
-    cb.credentials(|_url, username, allowed| cred(&mut tried_ssh_key, _url, username, allowed));
-
-    let mut opts = git2::FetchOptions::new();
-    opts.remote_callbacks(cb);
-    origin.fetch(&[branch_name], Some(&mut opts), None)?;
-
-    let remote_oid = repo.refname_to_id("FETCH_HEAD")?;
-    let head_oid = repo.refname_to_id("HEAD")?;
-
-    let (_, behind) = repo.graph_ahead_behind(head_oid, remote_oid)?;
-
-    if behind == 0 {
-        return Ok(());
-    }
-
-    let remote_annotated_commit = repo.find_annotated_commit(remote_oid)?;
-    let remote_commit = repo.find_commit(remote_oid)?;
-    repo.merge(&[&remote_annotated_commit], None, None)?;
-
-    //commit it
-    let mut index = repo.index()?;
-    let oid = index.write_tree()?;
-    let signature = repo.signature()?;
-    let parent_commit = find_last_commit(&repo)?;
-    let tree = repo.find_tree(oid)?;
-    let message = "pull and merge by ripasso";
-    let _commit = repo.commit(
-        Some("HEAD"), //  point HEAD to our new commit
-        &signature,   // author
-        &signature,   // committer
-        message,      // commit message
-        &tree,        // tree
-        &[&parent_commit, &remote_commit],
-    )?; // parents
-
-    //cleanup
-    repo.cleanup_state()?;
-    Ok(())
-}
-
 /// Import the key_ids from the signature file from a keyserver.
 /// # Errors
 /// Returns an `Err` if the download fails
@@ -1378,97 +1030,6 @@ pub fn pgp_import(store: &mut PasswordStore, text: &str, config_path: &Path) ->
     store.crypto.import_key(text, config_path)
 }
 
-fn triple<T: Display>(
-    e: &T,
-) -> (
-    Result<DateTime<Local>>,
-    Result<String>,
-    Result<SignatureStatus>,
-) {
-    (
-        Err(Error::GenericDyn(format!("{}", e))),
-        Err(Error::GenericDyn(format!("{}", e))),
-        Err(Error::GenericDyn(format!("{}", e))),
-    )
-}
-
-fn read_git_meta_data(
-    base: &Path,
-    path: &Path,
-    repo: &git2::Repository,
-    store: &PasswordStore,
-) -> (
-    Result<DateTime<Local>>,
-    Result<String>,
-    Result<SignatureStatus>,
-) {
-    let path_res = path.strip_prefix(base);
-    if let Err(e) = path_res {
-        return triple(&e);
-    }
-
-    let blame_res = repo.blame_file(path_res.unwrap(), None);
-    if let Err(e) = blame_res {
-        return triple(&e);
-    }
-    let blame = blame_res.unwrap();
-    let id_res = blame
-        .get_line(1)
-        .ok_or(Error::Generic("no git history found"));
-
-    if let Err(e) = id_res {
-        return triple(&e);
-    }
-    let id = id_res.unwrap().orig_commit_id();
-
-    let commit_res = repo.find_commit(id);
-    if let Err(e) = commit_res {
-        return triple(&e);
-    }
-    let commit = commit_res.unwrap();
-
-    let time = commit.time();
-    let time_return = Ok(Local.timestamp(time.seconds(), 0));
-
-    let name_return = name_from_commit(&commit);
-
-    let signature_return = verify_git_signature(repo, &id, store);
-
-    (time_return, name_return, signature_return)
-}
-
-fn verify_git_signature(
-    repo: &Repository,
-    id: &Oid,
-    store: &PasswordStore,
-) -> Result<SignatureStatus> {
-    let (signature, signed_data) = repo.extract_signature(id, Some("gpgsig"))?;
-
-    let signature_str = str::from_utf8(&signature)?.to_owned();
-    let signed_data_str = str::from_utf8(&signed_data)?.to_owned();
-
-    if store.valid_gpg_signing_keys.is_empty() {
-        return Err(Error::Generic(
-            "signature not checked as PASSWORD_STORE_SIGNING_KEY is not configured",
-        ));
-    }
-    match store.crypto.verify_sign(&signed_data_str.into_bytes(), &signature_str.into_bytes(), &store.valid_gpg_signing_keys) {
-        Ok(r) => Ok(r),
-        Err(VerificationError::InfrastructureError(message)) => Err(Error::GenericDyn(message)),
-        Err(VerificationError::SignatureFromWrongRecipient) => Err(Error::Generic("the commit wasn't signed by one of the keys specified in the environmental variable PASSWORD_STORE_SIGNING_KEY")),
-        Err(VerificationError::BadSignature) => Err(Error::Generic("Bad signature for commit")),
-        Err(VerificationError::MissingSignatures) => Err(Error::Generic("Missing signature for commit")),
-        Err(VerificationError::TooManySignatures) => Err(Error::Generic("If a git commit contains more than one signature, something is fishy")),
-    }
-}
-
-/// Initialize a git repository for the store.
-/// # Errors
-/// Returns an `Err` if the git init fails
-pub fn init_git_repo(base: &Path) -> Result<git2::Repository> {
-    Ok(git2::Repository::init(base)?)
-}
-
 /// Return a list of all passwords whose name contains `query`.
 pub fn search(store: &PasswordStore, query: &str) -> Vec<PasswordEntry> {
     let passwords = &store.passwords;
