@@ -741,7 +741,7 @@ fn read_config_default_path_in_env_var_with_pgp_setting() -> Result<()> {
 
     writeln!(
         &file,
-        "[stores.default]\npath = \"{}\"\nvalid_signing_keys = \"7E068070D5EF794B00C8A9D91D108E6C07CBC406\"\npgp_implementation = 'gpg'\nown_fingerprint = \"7E068070D5EF794B00C8A9D91D108E6C07CBC406\"",
+        "[stores.default]\npath = \"{}\"\nvalid_signing_keys = \"7E068070D5EF794B00C8A9D91D108E6C07CBC406\"\npgp_implementation = \"gpg\"\nown_fingerprint = \"7E068070D5EF794B00C8A9D91D108E6C07CBC406\"",
         dir.path().join(".password-store").to_str().unwrap()
     )?;
     file.flush()?;
@@ -797,10 +797,13 @@ fn save_config_one_store() {
     let config = std::fs::read_to_string(config_file.path()).unwrap();
 
     assert!(config.contains("[stores.\"s1 name\"]\n"));
-    assert!(config.contains(&format!("path = '{}'\n", passdir.path().display())));
-    assert!(config.contains(&format!("style_path = '{}'\n", style_file.path().display())));
-    assert!(config.contains("pgp_implementation = 'sequoia'\n"));
-    assert!(config.contains("own_fingerprint = '0000000000000000000000000000000000000000'\n"));
+    assert!(config.contains(&format!("path = \"{}\"\n", passdir.path().display())));
+    assert!(config.contains(&format!(
+        "style_path = \"{}\"\n",
+        style_file.path().display()
+    )));
+    assert!(config.contains("pgp_implementation = \"sequoia\"\n"));
+    assert!(config.contains("own_fingerprint = \"0000000000000000000000000000000000000000\"\n"));
 }
 
 #[test]
@@ -827,8 +830,8 @@ fn save_config_one_store_with_pgp_impl() {
     let data = fs::read_to_string(dir.path().join("file.toml")).unwrap();
 
     assert!(data.contains("[stores.default]"));
-    assert!(data.contains("pgp_implementation = 'gpg'"));
-    assert!(data.contains(&format!("path = '{}'\n", &dir.path().display())));
+    assert!(data.contains("pgp_implementation = \"gpg\""));
+    assert!(data.contains(&format!("path = \"{}\"\n", &dir.path().display())));
 }
 
 #[test]
@@ -855,9 +858,9 @@ fn save_config_one_store_with_fingerprint() {
     let data = fs::read_to_string(dir.path().join("file.toml")).unwrap();
 
     assert!(data.contains("[stores.default]"));
-    assert!(data.contains("pgp_implementation = 'sequoia'"));
-    assert!(data.contains("own_fingerprint = '7E068070D5EF794B00C8A9D91D108E6C07CBC406'"));
-    assert!(data.contains(&format!("path = '{}'\n", &dir.path().display())));
+    assert!(data.contains("pgp_implementation = \"sequoia\""));
+    assert!(data.contains("own_fingerprint = \"7E068070D5EF794B00C8A9D91D108E6C07CBC406\""));
+    assert!(data.contains(&format!("path = \"{}\"\n", &dir.path().display())));
 }
 
 #[test]
@@ -1486,8 +1489,11 @@ fn test_format_error() {
         "configuration is frozen"
     );
     assert_eq!(
-        format!("{}", Error::from(toml::ser::Error::DateInvalid)),
-        "a serialized date was invalid"
+        format!(
+            "{}",
+            Error::from(toml::ser::to_string_pretty(&None::<String>).err().unwrap())
+        ),
+        "unsupported None value"
     );
     assert_eq!(
         format!("{}", Error::from("custom error message")),
